#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <X11/Xlib.h>
#include <GL/glx.h>
#include <GL/gl.h>



int main(int argc, char *argv[])
{
    Display *pDisplay = NULL;
    int i, nScreen;
    XVisualInfo *pVI = NULL;
    GLXContext glxctx = NULL;
    Pixmap pix;
    GLXPixmap glpix;
    char   *dpy = NULL;
    int bVerbose = 0;
    int bSynch = 0;

    for(i=1; i < argc; i++)
    {
        int nArgLen = strlen(argv[i]);
   	if(!strncmp(argv[i],"-display",nArgLen))
	{
	    if( ++i < argc )
	    {
	        dpy = argv[i];
	    }
	}
   	else if(!strncmp(argv[i],"-verbose",nArgLen))
	    bVerbose = 1;
   	else if(!strncmp(argv[i],"-synchronize",nArgLen))
	    bSynch = 1;
	else
	    fprintf( stderr, "Unknown option ignored: %s.\n", argv[i] );
    }

    int anAttribs[] =
    {
	GLX_DOUBLEBUFFER,
	GLX_LEVEL, 0,
	GLX_RGBA,
	GLX_RED_SIZE, 1,
	GLX_GREEN_SIZE, 1,
	GLX_BLUE_SIZE, 1,
	GLX_DEPTH_SIZE, 12,
	0,
    };

/*
    fprintf(stderr,"unsigned int   %d\n"
	           "unsigned long  %d\n"
	           "Window         %d\n"
	           "pixmap         %d\n"
		   "GLXPixmap      %d\n"
		   "GLXContext     %d\n",
	    sizeof(unsigned int), sizeof(unsigned long), 
	    sizeof(Window), sizeof(Pixmap),
	    sizeof(GLXPixmap), sizeof(GLXContext) );
    fflush(stderr);
    assert( sizeof(Pixmap) == sizeof(unsigned long) );
    assert( sizeof(GLXPixmap) == sizeof(unsigned long) );
    assert( sizeof(GLXContext) == sizeof(unsigned long) );
*/

    /* Open display */
    pDisplay = XOpenDisplay(dpy);
    if( ! pDisplay ) 
    {
	fprintf( stderr, "Could not connect to %s.\n", XDisplayName(dpy) );
	exit(1);
    }
    if( bSynch )
	XSynchronize(pDisplay,True);
    if( bVerbose )
	fprintf( stderr, "Connected to display: %s\n", XDisplayName(dpy) );

    if( False == glXQueryExtension(pDisplay, NULL, NULL) ) 
    {
	fprintf( stderr, "OpenGL not supported by X server.\n");
	exit(1);
    }
    if( bVerbose )
	fprintf( stderr, "GLX is supported.\n" );

    nScreen = DefaultScreen( pDisplay );
    if( bVerbose )
	fprintf( stderr, "Screen is %d.\n", nScreen );

    /* Find a single-buffered visual */
    pVI = glXChooseVisual( pDisplay, nScreen, &anAttribs[1] );
    if( ! pVI ) 
    {
	pVI = glXChooseVisual( pDisplay, nScreen, anAttribs );
	if( pVI && bVerbose )
	    fprintf( stderr, "Found a double-buffered visual.\n", nScreen );
    }
    else if( bVerbose )
	fprintf( stderr, "Found a single-buffered visual.\n", nScreen );
    if( ! pVI ) 
    {
	fprintf( stderr, "Could not find a GL visual.\n");
	exit(1);
    }
    if(!((pVI->class==DirectColor) || (pVI->class==TrueColor)))
    {
	fprintf( stderr, "Wanted a DirectColor or TrueColor visual.\n");
	exit(1);
    }


    /* Create an OpenGL context */
    glxctx = glXCreateContext( pDisplay, pVI, NULL, False );
    if( ! glxctx ) 
    {
	fprintf( stderr, "Unable to create a GL context.\n");
	exit(1);
    }
    if( bVerbose )
	fprintf( stderr, "Created a GLX context.\n" );

    pix = XCreatePixmap( pDisplay, RootWindow(pDisplay, nScreen), 100, 100, pVI->depth);
    if(!pix)
    {
	fprintf( stderr, "Unable to create a pixmap.\n");
	exit(1);
    }
    if( bVerbose )
	fprintf( stderr, "Created an X pixmap.\n" );

    glpix = glXCreateGLXPixmap( pDisplay, pVI, pix );
    if(!glpix)
    {
	fprintf( stderr, "Unable to create a GLX pixmap.\n");
	exit(1);
    }
    if( bVerbose )
	fprintf( stderr, "Created a GLX pixmap.\n" );

    if( ! glXMakeCurrent(pDisplay, glpix, glxctx) )
    {
	if( glxctx != glXGetCurrentContext() )
	{
	    fprintf( stderr, "Unable to make GLX pixmap context current.\n");
	    exit(1);
	}
	else if( bVerbose )
	    fprintf( stderr, "Made GLX pixmap context current but got error.\n" );
    }
    else if( bVerbose )
	fprintf( stderr, "Made GLX pixmap context current.\n" );

    XSync( pDisplay, False );

    /* Release everything */
    glXMakeCurrent( pDisplay, None, NULL );
    if( bVerbose )
	fprintf( stderr, "Released GLX context.\n" );
    glXDestroyGLXPixmap( pDisplay, glpix );
    if( bVerbose )
	fprintf( stderr, "Destroyed GLX pixmap.\n" );
    XFreePixmap( pDisplay, pix );
    if( bVerbose )
	fprintf( stderr, "Destroyed X pixmap.\n" );
    glXDestroyContext( pDisplay, glxctx );
    if( bVerbose )
	fprintf( stderr, "Destroyed GLX context.\n" );

    return 0;
}

